var ListagemDatasets = SuperWidget.extend({
    //Variáveis da widget
    documentId: null,
    documentVersion: null,
    datasetId: null,
    parentId: null,
    camposTela: null,
    
    dataInit: null,
    //Método iniciado quando a widget é carregada
    init: function() {
    	//Executa se não for modo de edição da widget
    	if(!this.isEditMode){    		
    		/**
    		 * Verifica se os campos foram preenchidos no modo de edição da widget, e caso sim, inicia as variáveis com os valores informados no modo de edição da widget (edit.ftl).
    		 * Caso contrário apresenta mensagem de erro solicitante a verificação das informações
    		 */    	
    		if ((this.datasetId != "" && this.datasetId != null && this.datasetId != undefined) 
    				&& (this.parentId != "" && this.parentId != null && this.parentId != undefined)) {    		
    			ListagemDatasets.datasetId = this.datasetId;
    			ListagemDatasets.parentId = this.parentId;
    		} else {
    			FLUIGC.toast({
    				message : "Verifique os campos 'Nome do Dataset do Fichário' e 'Número do Fichário' no modo de edição da widget!",
    				type : 'danger'
    			});
    			return false;
    		}
    	}
    	
    	//Chamada à função para carregar o datatable
    	this.loadDadosDataset();
    	
    	//Após carregamento completo do datatable, desabilita os botões de edição e exclusão
    	$('#dados_dataset').on('fluig.datatable.loadcomplete', function() {
            $(".listagem_datasets_widget button#datatable-edit-row").attr("disabled", "disabled");
            $(".listagem_datasets_widget button#datatable-del-row").attr("disabled", "disabled");
        });
    	
    	//Ao selecionar uma linha do datatable, habilita os botões de edição e exclusão
        $('.listagem_datasets_widget #dados_dataset').on('fluig.datatable.onselectrow', function() {
            $(".listagem_datasets_widget button#datatable-edit-row").removeAttr("disabled");
            $(".listagem_datasets_widget button#datatable-del-row").removeAttr("disabled");
        });
    		       
        //Ao abrir a modal de edição do registro, altera o css em relação a altura e largura
    	$(document).on('shown.bs.modal','#modal_registro', function () {
	       $(this).find('.modal-body').css({
	              width:'auto', 
	              height:'auto',
	              'max-height':'100%'
	       });
    	});
    },
  
    //BIND de eventos nos clicks dos botões da widget
    bindings: {
        local: {
        	'datatable-add-row': ['click_addRow'],
            'datatable-edit-row': ['click_editRow'],
            'datatable-del-row': ['click_deleteCard'],
            'save-params' : [ 'click_saveParams' ],
            'load-registros' : ['click_loadDadosDataset']
        },
        global: {
        	'save-modal': ['click_updateCard'],
        	'save-modal-inclusao': ['click_createCard']
        }
    },
    
    /**
     * Retorna o parâmetro da URL recebido como argumento
     */
    getUrlParameter: function(sParam) {
		let sPageURL = window.location.search.substring(1),
			sURLVariables = sPageURL.split('&'),
			sParameterName;

		for (var i = 0; i < sURLVariables.length; i++) {
			sParameterName = sURLVariables[i].split('=');

			if (sParameterName[0] === sParam) {
				return sParameterName[1] === undefined ? true : decodeURIComponent(sParameterName[1]);
			}
		}
	},
    
	/**
	 * Salva os dados informados no modo de edição da widget
	 */
    saveParams : function(element, event) {
    	let parentId = $('#parentId_' + this.instanceId).val();
    	let datasetId = $('#datasetId_' + this.instanceId).val();
    	let camposTela = $('#camposTela_' + this.instanceId).val();
    	let titulos = $('#titulos_' + this.instanceId).val();
    	
    	if(parentId == "" || datasetId == "" || camposTela == "" || titulos == ""){
    		FLUIGC.toast({
		        message: "Todos os campos são de preenchimento obrigatório!",
		        type: 'warning'
		    });
    		
    		return;
    	}
    	
		let preferences = {};
		preferences.parentId = parentId;
		preferences.datasetId = datasetId;
		preferences.camposTela = camposTela;
		preferences.titulos = titulos;
		WCMSpaceAPI.PageService.UPDATEPREFERENCES({
			async : true,
			success : function(data) {
				FLUIGC.toast({
					message : data.message,
					type : 'success'
				});
			},
			fail : function(xhr, message, errorData) {
				FLUIGC.toast({
					message : errorData.message,
					type : 'danger'
				});
			}
		}, this.instanceId, preferences);
	},    
    
	/**
	 * Envia o registro selecionado para a lixeira
	 */
    deleteCard: function(){
    	let that = this;
    	
    	//Mensagem para confirmação para exclusão do registro 
    	FLUIGC.message.confirm({
    	    message: 'Deseja enviar o(s) documento(s) selecionado(s) para a lixeira?',
    	    title: 'Remover documento',
    	    labelYes: 'Remover',
    	    labelNo: 'Cancelar'
    	}, function(result, el, ev) {
    		if(result){
    			//Caso mensagem seja confirmada, chama a função para excluir o registro
    			that.deleteDocument();
    		}
    	});
    },
    
    /**
     * Deleta o registro através da API pública 'deleteDocument'
     */
    deleteDocument: function(){
    	let that = this;
    	let row = this.myTable.getRow(this.myTable.selectedRows()[0]);
        let documentId = row.documentid;
        
    	$.ajax({
    		type : "POST",
    		contentType: "application/json",
    		url : '/api/public/2.0/documents/deleteDocument/'+documentId,
    		error: function(e) {
    			FLUIGC.toast({
					title: '',
					message: e.message,
					type: 'danger'
				});
    		},
    		success: function(data){
    			FLUIGC.toast({
					title: '',
					message: "Ficha " + documentId + " removida com sucesso",
					type: 'success'
				});
    			that.loadDadosDataset();
    		},
    	});
    },
    
    /**
     * Cria o registro de formulário através da API pública 'create'
     */    
    createCard: function(){
    	let that = this;
    	let parentId = that.parentId;
    	let constraints = this.getAllFormFields();
    	
    	//JSON necessário para criação de uma ficha 
    	var request_data = {
			"documentDescription": "Criado via widget",
			"parentDocumentId": parentId,
			"version": 1000,
			"inheritSecurity" : true,
			"formData" : constraints,
			"attachments" : []
		};
    	
    	$.ajax({
            type : 'POST',
            dataType : 'json',
            contentType : 'application/json',
            url : '/api/public/2.0/cards/create',
            data : JSON.stringify(request_data),
            success : function(data, status, xhr) {
                if (data) {
                	$("button.close").click();
                	that.loadDadosDataset();
                }
            },
            error : function(xhr, status, error) {
                FLUIGC.toast({
                    message : xhr.responseJSON.message.message,
                    type : "danger"
                });
            }
        });
    },
    
    /**
     * Atualiza o registro de formulário através da API 'editCard'
     */
    updateCard : function(){
    	let that = this;
    	let documentId = ListagemDatasets.documentId;
    	let documentVersion = ListagemDatasets.documentVersion;
    	let constraints = this.getAllFormFields();
    	
    	//JSON necessário para edição de uma ficha
    	var paramData = {
    			"companyId" : WCMAPI.getTenantId(),
    			"documentId" : documentId,
    			"version" : documentVersion,
    			"cardDescription" : "",
    			"documentDescription" : "",
    			"publisherId" : "onboardingapp",
    			"inheritSecurity" : "true",
    			"documentType" : "5",
    			"approved" : "true",
    			"principalFileName" : "",
    			"hasParentApprover" : "false",
    			"permissionType" : 0,
    			"restrictionType" : 0,
    			"userSecurityLevel" : 3,
    			"cardFormData" : constraints
    	}
    	
    	$.ajax({
            type : 'POST',
            dataType : 'json',
            contentType : 'application/json',
            url : '/ecm/api/rest/ecm/cardPublisher/editCard',
            data : JSON.stringify(paramData),
            success : function(data, status, xhr) {
                if (data) {
                	$("button.close").click()
                	that.loadDadosDataset();
                }
            },
            error : function(xhr, status, error) {
                FLUIGC.toast({
                    message : xhr.responseJSON.message.message,
                    type : "danger"
                });
            }
        });
    },
    
    /**
     * Inicia o datatable baseado nos dados informados no modo de edição,
     * realizando também algumas configurações necessárias; 
     */
    loadDadosDataset: function(){
    	let that = this;
    	
    	//criação das variáveis locais para recuperação dos dados do fichário em questão
    	let datasetId = that.datasetId;
    	let retornoDataset = [];
    	
    	//recupera valor informado no campo 'Limitar' 
    	let limit = $("#limite_registros").val();
    	
    	//caso valor do campo 'Limitar' não tenha sido informado, utiliza 100 como padrão
    	if(limit == "" || limit == undefined){
    		limit = 100;
    	}
    	
    	//url da api para recuperar os dados do dataset do fichário
    	let url = '/api/public/ecm/dataset/search?datasetId=' + datasetId + '&likeField=metadata_active,true&limit=' + limit;
    	
    	//chamada à api informada acima
    	$.ajax({
    		type: 'GET',
    		async: false,
    		dataType: 'json',
    		url: url,
    		contentType: 'application/json',
    		success: function(data, status, xhr){
    			//validação se há retorno de dados
    			if (data != null && data != undefined && data.content.length > 0) {
    				//variável recebe o valor dos dados retornados
    				retornoDataset = data.content;
    			}
    		},
    		error: function(xhr, status, error){
    			FLUIGC.toast({
    				message : "Erro: " + error.message,
    				type: "danger"
    			})
    		}
    	});
    	
    	/**
    	 * Início da seção que define o nome dos campos que aparecerão no cabeçalho do datatable e
    	 * também os campos em si para retorno dos dados nas linhas  
    	 */
    	let campos = that.camposTela.split(",");
    	let nomes = that.titulos.split(",");
    	
    	let tituloCampos = [];
    	for (var i = 0; i < nomes.length; i++) {
    		let registro = {"title": nomes[i]};
    		tituloCampos.push(registro);
    	}
    	
    	//inicialização do datatable com as informações do formulário em questão
	    that.myTable = FLUIGC.datatable('#dados_dataset', {
	        dataRequest: retornoDataset,
	        renderContent: campos,
	        header: tituloCampos,
	        ordering: "true",
	        search: {
	            enabled: true,
	            onlyEnterkey: true,
	            searchAreaStyle: 'col-md-3',
	            onSearch: function(res) {
	            	//realiza a busca novamente via API quando pressionada a tecla 'Enter' ou clicado na lupa, levando em conta os filtros informados
                	let campoFiltro = $("#sl_filtro_busca option:selected").val();
	                let url = '/api/public/ecm/dataset/search?datasetId=' + datasetId + '&likeField=' + campoFiltro + '&likeValue=' + res;
                	
	                retornoDataset = [];
                	$.ajax({
                		type: 'GET',
                		async: false,
                		dataType: 'json',
                		url: url,
                		contentType: 'application/json',
                		success: function(data, status, xhr){
                			if (data != null && data != undefined && data.content.length > 0) {
                				retornoDataset = data.content;
                			}
                		},
                		error: function(xhr, status, error){
                			FLUIGC.toast({
                				message : "Erro: " + error.message,
                				type: "danger"
                			})
                		}
                	});
                	
                	//atualiza o datatable com os novos dados buscados
                    that.myTable.reload(retornoDataset);
	            }
	        },
	        actions: {
	            enabled: true,
	            template: '.template_area_buttons',
	            actionAreaStyle: 'col-md-9'
	        },
	        navButtons: {
	            enabled: false,
	        },
	        tableStyle: 'table-striped'
	    }, function(err, data) {
	        if(data) {
	            dataInit = data;
	        }
	        else if (err) {
	            FLUIGC.toast({
	                message: err.message,
	                type: 'danger'
	            });
	        }
	    });

		//no campo de busca, após uma tecla ser solta (keyup) realiza o filtro em tela do valor informado, sem chamar novamente a API de busca
	    $('#fluig-data-table-input').keyup(function() {
	    	let campos = that.camposTela.split(",");
	    	let nomes = that.titulos.split(",");
	    	
	    	let tituloCampos = [];
	    	for (var i = 0; i < nomes.length; i++){
	    		tituloCampos.push({"title": nomes[i]});
	    	}
	    	
	        let fieldValue = this.value;
	        let dataAll = that.myTable.getData();
	        let retorno = [];
	        
	        let search = dataInit.filter(function(el) {
		    	for (var i = 0; i < campos.length; i++) {
		    		if (el[campos[i]].toString().toUpperCase().indexOf(fieldValue.toString().toUpperCase()) >= 0) {
		    			return el;
		    		}
		    	}
	        });
	            
	        //Se houve resultados, atualiza o datatable com os valores encontrados. Caso contrário, apresenta uma mensagem 'Sem resultados'
	        if (search && search.length) {
	            that.myTable.reload(search);
	        } else {
	            FLUIGC.toast({
	                title: 'Busca: ',
	                message: 'Sem resultados',
	                type: 'success'
	            });
	        }
	    });
		    
		//Adiciona nas opções do campo 'Campo Filtro' (select) todos os campos do cabeçalho do datatable
		for (var i = 0; i < campos.length; i++){
			$('#sl_filtro_busca').append($("<option></option>").attr("value", campos[i]).text(nomes[i])); 
		}
	},
    
	/**
	 * Abre a modal para edição do registro de formulário
	 */
    editRow: function(el, ev) {
    	let row = this.myTable.getRow(this.myTable.selectedRows()[0]);
    	let documentId = "";
        let documentVersion = "";
        let parentId = this.parentId;
    	if (row == undefined) {
    		documentId = this.getUrlParameter("nr_ficha");
    		documentVersion = this.getUrlParameter("versao");
    	} else {
    		documentId = row.documentid;
    		documentVersion = row.version;
    	}
        
        ListagemDatasets.documentId = documentId;
        ListagemDatasets.documentVersion = documentVersion;
        var url = "/webdesk/streamcontrol/" + parentId + "/" + documentId + "/" + documentVersion + "/?WDCompanyId=" + WCMAPI.getTenantId() + "&WDNrDocto=" + documentId+"&WDNrVersao=" + documentVersion+"&WDParentDocumentId=" + parentId+"&edit=true";
       
    	var myModal = FLUIGC.modal({
    	    title: 'Edição',
    	    content: '<h1>Modal Content</h1>',
    	    id: 'modal_registro',
    	    size: 'full',
    	    actions: [{
    	        'label': 'Salvar',
    	        'bind': 'data-save-modal',
    	    },{
    	        'label': 'Fechar',
    	        'autoClose': true
    	    }]
    	},
    	function(err, data) {
    	    if(err) {
    	        // do error handling
    	    } else {
    	    	$(".modal-body").html('<iframe width="100%" src='+url+' frameborder="0" onload="heightFrame()" id="print_frame"></iframe>');
    	    	$("#modal_registro").css("padding",'0px');
   	 	   		$("#modal_registro").css("margin",'0px');
   	 	   		$("#modal_registro .modal-content").css("min-height","100vh");
   	 	   		$("#modal_registro").addClass("width_important");
   	 	   		$("#modal_registro .modal-body").css("min-height","80vh");
   	 	   		$("#modal_registro .modal-body").addClass("no_overflow");
   	 	   		$("#print_frame").css("min-height","85vh");
   	 	   		$("#print_frame").addClass("overflow_auto");
    	    }
    	});
    },
    
    /**
	 * Abre a modal para inclusão do registro de formulário
	 */
    addRow: function(){
    	let that = this;
    	let parentId = that.parentId;
    	
        let url = "/webdesk/streamcontrol/" + parentId + "/0/0/?WDCompanyId=" + WCMAPI.getTenantId() + "&WDNrDocto=0&WDNrVersao=0&WDParentDocumentId=" + parentId;
    	
    	let myModal = FLUIGC.modal({
    	    title: 'Inclusão',
    	    content: '<h1>Modal Content</h1>',
    	    id: 'modal_registro_inclusao',
    	    size: 'full',
    	    actions: [{
    	        'label': 'Salvar',
    	        'bind': 'data-save-modal-inclusao',
    	    },{
    	        'label': 'Fechar',
    	        'autoClose': true
    	    }]
    	}, function(err, data) {
    	    if(err) {
    	        // do error handling
    	    } else {
    	    	$(".modal-body").html('<iframe width="100%" src='+url+' frameborder="0" onload="heightFrame()" id="print_frame"></iframe>');
    	    	$("#modal_registro_inclusao").css("padding",'0px');
   	 	   		$("#modal_registro_inclusao").css("margin",'0px');
   	 	   		$("#modal_registro_inclusao .modal-content").css("min-height","100vh");
   	 	   		$("#modal_registro_inclusao").addClass("width_important");
   	 	   		$("#modal_registro_inclusao .modal-body").css("min-height","80vh");
   	 	   		$("#modal_registro_inclusao .modal-body").addClass("no_overflow");
   	 	   		$("#print_frame").css("min-height","85vh");
   	 	   		$("#print_frame").addClass("overflow_auto");
    	    }
    	});
    },
    
    /**
     * Retorna os valores de todos os campos do tipo 'radio' do registro de formulário, tanto na edição quando inclusão
     */
    returnRadioValue : function(radioName){
		return $("#print_frame").contents().find('input[name='+radioName).val() == undefined ? null : $("#print_frame").contents().find('input[name='+radioName).val();
	},
	
	/**
     * Retorna os valores de todos os campos do tipo 'checkbox' do registro de formulário, tanto na edição quando inclusão
     */
	returnCheckboxValue : function(checkboxId){
		let iframe = document.getElementById("print_frame");
		return $("#"+checkboxId,iframe.contentWindow.document).prop("checked") == true ? "on" : ""; 
	},
    
	/**
     * Retorna os valores de todos os campos do registro de formulário, tanto na edição quando inclusão
     */
    getAllFormFields : function(){
    	let that = this;
    	let constraints = [];
    	
    	let iframe = document.getElementById("print_frame");
    	let elements = iframe.contentWindow.document.querySelectorAll("input[type=text][name],input[type=hidden][name],textarea[name],input[type=datetime-local]");
    	
    	let inputsText = iframe.contentWindow.document.querySelectorAll("input[type=text][name],input[type=hidden][name],textarea[name],input[type=datetime-local],input[type=date");
    	for (var i = 0; i < inputsText.length; i++){
    		constraints.push({
    			"name":  inputsText[i].name, 
    			"value": inputsText[i].value
			});
    	}
    	
    	let inputsCheckbox = iframe.contentWindow.document.querySelectorAll("input[type=checkbox][name]");
    	for (var i = 0; i < inputsCheckbox.length; i++) {
    		let value = that.returnCheckboxValue(inputsCheckbox[i].id);
    		constraints.push({
    			"name":  inputsCheckbox[i].name, 
    			"value": value
			});
    	}
    	
    	let inputsRadio = iframe.contentWindow.document.querySelectorAll("input[type=radio][name]");
    	for (var i = 0; i < inputsRadio.length; i++){
    		let value = that.returnRadioValue(inputsRadio[i].name);
    		let name = inputsRadio[i].name;

    		if (value != null && value != "null" && value != undefined) {
    			constraints.push({
    				"name" : name, 
    				"value" : value
				});
    		}
    	}
    	
    	let selects = iframe.contentWindow.document.querySelectorAll("select");
    	for (var i = 0; i < selects.length; i++) {
    		let name = selects[i].name;
    		if (selects[i].options[selects[i].selectedIndex] != undefined) {
    		    var value = selects[i].options[selects[i].selectedIndex].value;

				if(value != null && value != "null" && value != undefined){
					constraints.push({
						"name" : name,
						"value" : value
					});
				}	
    		}
    	}

    	return constraints;
    }
});